@@ -2,6 +2,8 @@
 (* license: GNU Lesser General Public License Version 2.1 or later           *)
 (* ------------------------------------------------------------------------- *)
 
+open Elpi_util
+
 open API
 open RawData
 open Utils
@@ -40,7 +42,7 @@ let register_eval, register_eval_ty, lookup_eval, eval_declaration =
  (fun nargs (s,tys) f ->
    tys |> List.iter (fun ty ->
      let ty =
-       if nargs = -2 then
+       if nargs < 0 then
          Printf.sprintf "type (%s) %s." s (str_of_ty (abs nargs) ty)
        else
          Printf.sprintf "type %s %s." s (str_of_ty (abs nargs) ty) in
@@ -107,7 +109,7 @@ let _ =
    | [ CData x; CData y ] when ty2 string x y ->
          of_string (to_string x ^ to_string y)
    | _ -> type_error "Wrong arguments to ^") ;
-  register_evals 1 [ "~",["int";"float"] ; "i~",["int"] ; "r~",["float"] ] (function
+  register_evals ~-1 [ "~",["int";"float"] ; "i~",["int"] ; "r~",["float"] ] (function
    | [ CData x ] when is_int x -> (morph1 int (~-) x)
    | [ CData x ] when is_float x -> (morph1 float (~-.) x)
    | _ -> type_error "Wrong arguments to ~/i~/r~") ;
@@ -738,11 +740,9 @@ let lp_builtins = let open BuiltIn in let open BuiltInData in [
     In(string, "S",
     Out(any,   "T",
     Full(ContextualConversion.unit_ctx, "parses a term T from S"))),
-  (fun s _ ~depth () () state ->
+  (fun text _ ~depth () () state ->
      try
-       let loc = Ast.Loc.initial "(string_of_term)" in
-       let t = Parse.goal loc s in
-       let state, t = Quotation.term_at ~depth state t in
+       let state, t = Quotation.term_at ~depth state text in
        state, !:t, []
      with
      | Parse.ParseError _ -> raise No_clause)),
@@ -751,13 +751,11 @@ let lp_builtins = let open BuiltIn in let open BuiltInData in [
   MLCode(Pred("readterm",
     In(in_stream, "InStream",
     Out(any,      "T",
-    Full(ContextualConversion.unit_ctx, "reads T from InStream"))),
+    Full(ContextualConversion.unit_ctx, "reads T from InStream, ends with \\n"))),
   (fun (i,source_name) _ ~depth () () state ->
      try
-       let loc = Ast.Loc.initial source_name in
-       let strm = Stream.of_channel i in
-       let t = Parse.goal_from_stream loc strm in
-       let state, t = Quotation.term_at ~depth state t in
+       let text = input_line i in
+       let state, t = Quotation.term_at ~depth state text in
        state, !:t, []
      with
      | Sys_error msg -> error msg
@@ -813,9 +811,9 @@ counter C N :- trace.counter C N.|};
           ~file_resolver:(Parse.std_resolver ~paths:[] ())
           () in
       try
-        let ap = Parse.program ~elpi [f] in
+        let ap = Parse.program ~elpi ~files:[f] in
         let loc = Ast.Loc.initial "(quote_syntax)" in
-        let aq = Parse.goal loc s in
+        let aq = Parse.goal ~elpi ~loc ~text:s in
         let p = Compile.(program ~flags:default_flags ~elpi [ap]) in
         let q = API.Compile.query p aq in
         let state, qp, qq = Quotation.quote_syntax_runtime state q in
@@ -1084,14 +1082,14 @@ X == Y :- same_term X Y.
   DocNext);
 
   MLCode(Pred("occurs",
-    In(any, "a constant (global or eigenvariable)",
+    In(any, "an atom, that is a global constant or a bound name (aka eigenvariable)",
     In(any, "a term",
-    Easy     "checks if the constant occurs in the term")),
+    Easy     "checks if the atom occurs in the term")),
   (fun t1 t2 ~depth ->
      let occurs_in t2 t =
        match look ~depth t with
        | Const n -> occurs n depth t2
-       | _ -> error "The second argument of occurs must be a constant" in
+       | _ -> error "The second argument of occurs must be an atom" in
      if occurs_in t2 t1 then () else raise No_clause)),
   DocNext);
 
@@ -1192,14 +1190,14 @@ if2 _  _  _  _  E :- !, E. |};
 ]
 ;;
 
-let elpi_stdlib_src = let open BuiltIn in let open BuiltInData in [ 
+let elpi_stdlib_src = let open BuiltIn in [ 
 
   LPCode Builtin_stdlib.code
 
 ]
 
-let ocaml_set ~name (type a)
-   (alpha : a Conversion.t) (module Set : Util.Set.S with type elt = a) =
+let ocaml_set_conv ~name (type a) (type b)
+   (alpha : a Conversion.t) (module Set : Util.Set.S with type elt = a and type t = b) =
  
 let set = OpaqueData.declare {
   OpaqueData.name;
@@ -1215,6 +1213,7 @@ let set = { set with Conversion.ty = Conversion.(TyName name) } in
 
 let open BuiltIn in let open BuiltInData in 
 
+set,
 [
   LPCode ("kind "^name^" type.");
 
@@ -1301,6 +1300,7 @@ let open BuiltIn in let open BuiltInData in
   DocAbove);
 ] 
 ;;
+let ocaml_set ~name c m = snd (ocaml_set_conv ~name c m)
 
 let ocaml_map ~name (type a)
    (alpha : a Conversion.t) (module Map : Util.Map.S with type key = a) =
@@ -1378,20 +1378,22 @@ let open BuiltIn in let open BuiltInData in
 ;;
 
 module LocMap : Util.Map.S with type key = Ast.Loc.t = Util.Map.Make(Ast.Loc)
-module LocSet : Util.Set.S with type elt = Ast.Loc.t = Util.Set.Make(Ast.Loc)
 
-let elpi_map =  let open BuiltIn in let open BuiltInData in [
+let elpi_map =  let open BuiltIn in [
   
     LPCode Builtin_map.code
     
 ]
 
-let elpi_set =  let open BuiltIn in let open BuiltInData in [
+let elpi_set =  let open BuiltIn in [
   
     LPCode Builtin_set.code
     
 ]
 
+let string_set, string_set_decl = ocaml_set_conv ~name:"std.string.set" BuiltInData.string (module API.Compile.StrSet)
+let int_set, int_set_decl = ocaml_set_conv ~name:"std.int.set"    BuiltInData.int    (module API.Utils.IntSet)
+let loc_set, loc_set_decl = ocaml_set_conv ~name:"std.loc.set"    BuiltInData.loc    (module API.Utils.LocSet) 
 
 let elpi_stdlib =
   elpi_stdlib_src @
@@ -1408,9 +1410,9 @@ let elpi_stdlib =
   ocaml_map ~name:"std.string.map" BuiltInData.string (module Util.StrMap) @ 
   ocaml_map ~name:"std.int.map"    BuiltInData.int    (module Util.IntMap) @ 
   ocaml_map ~name:"std.loc.map"    BuiltInData.loc    (module LocMap) @ 
-  ocaml_set ~name:"std.string.set" BuiltInData.string (module Util.StrSet) @ 
-  ocaml_set ~name:"std.int.set"    BuiltInData.int    (module Util.IntSet) @ 
-  ocaml_set ~name:"std.loc.set"    BuiltInData.loc    (module LocSet) @
+  string_set_decl @ 
+  int_set_decl @ 
+  loc_set_decl @
   []
 ;;
 
@@ -1422,7 +1424,7 @@ let ocaml_runtime = let open BuiltIn in let open BuiltInData in [
     In (string,"Name",
     Out(int,   "Value",
     Easy       "reads the Value of a trace point Name")),
-  (fun s _ ~depth:_ -> !:(Trace_ppx_runtime.Runtime.get_cur_step s))),
+  (fun s _ ~depth:_ -> !:(Trace_ppx_runtime.Runtime.get_cur_step ~runtime_id:0 s))),
   DocAbove);
 
   MLCode(Pred("gc.get",
@@ -1518,6 +1520,10 @@ let std_builtins =
 
 
 let default_checker () =
-  let elpi = API.Setup.init ~builtins:[std_builtins] () in
-  let ast = API.Parse.program_from_stream ~elpi (API.Ast.Loc.initial "(checker)") (Stream.of_string Builtin_checker.code) in
-  API.Compile.program ~flags:API.Compile.default_flags ~elpi [ast]
+  try
+    let elpi = API.Setup.init ~builtins:[std_builtins] () in
+    let ast = API.Parse.program_from ~elpi ~loc:(API.Ast.Loc.initial "(checker)") (Lexing.from_string Builtin_checker.code) in
+    API.Compile.program ~flags:API.Compile.default_flags ~elpi [ast]
+  with
+  | API.Parse.ParseError(loc,msg) -> API.Utils.anomaly ~loc msg
+  | API.Compile.CompileError(loc,msg) -> API.Utils.anomaly ?loc msg
\ No newline at end of file
