@@ -33,5 +33,5 @@ let () =
   let us = cc ~elpi ~flags 1 us in
   let ex = cc ~elpi ~flags 2 ex in
   let p = Compile.assemble ~elpi (us :: list_init 0 50000 (fun _ -> ex)) in
-  let q = Compile.query p (Parse.goal_from_stream (Ast.Loc.initial "g") (Stream.of_string "main")) in
+  let q = Compile.query p (Parse.goal_from ~elpi ~loc:(Ast.Loc.initial "g") (Lexing.from_string "main")) in
   exec q
