@@ -12,11 +12,12 @@ type j = J : (Format.formatter -> 'a -> unit) * 'a -> j
 val enter : runtime_id:int -> string ->  (Format.formatter -> unit) -> unit
 val info : runtime_id:int -> ?goal_id:int -> string -> j list -> unit
 val exit : runtime_id:int -> string -> bool -> exn option -> float -> unit
+val end_trace : runtime_id:int -> unit
 
 val set_cur_pred : string option -> unit
-val get_cur_step : string -> int
+val get_cur_step : runtime_id:int -> string -> int
 
-val log : string -> string -> int -> unit
+val log : runtime_id:int -> string -> string -> int -> unit
 
 val debug : bool ref
 val parse_argv : string list -> string list
